package javax.swing.plaf.basic;

import sun.swing.UIAction;
import javax.swing.AbstractAction;
import java.awt.event.ComponentAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import javax.swing.event.MouseInputAdapter;
import javax.swing.BoundedRangeModel;
import java.awt.event.FocusEvent;
import java.awt.event.ComponentEvent;
import javax.swing.event.ChangeEvent;
import java.beans.PropertyChangeEvent;
import java.awt.Polygon;
import javax.swing.Icon;
import java.awt.Toolkit;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import java.awt.Graphics;
import java.util.Enumeration;
import java.util.Dictionary;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import sun.swing.DefaultLookup;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.UIManager;
import java.awt.IllegalComponentStateException;
import javax.swing.LookAndFeel;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Color;
import java.beans.PropertyChangeListener;
import java.awt.event.FocusListener;
import java.awt.event.ComponentListener;
import javax.swing.event.ChangeListener;
import java.awt.Rectangle;
import java.awt.Insets;
import javax.swing.JSlider;
import javax.swing.Timer;
import javax.swing.plaf.SliderUI;

public class BasicSliderUI extends SliderUI
{
    private static final BasicSliderUI.Actions SHARED_ACTION;
    public static final int POSITIVE_SCROLL = 1;
    public static final int NEGATIVE_SCROLL = -1;
    public static final int MIN_SCROLL = -2;
    public static final int MAX_SCROLL = 2;
    protected Timer scrollTimer;
    protected JSlider slider;
    protected Insets focusInsets;
    protected Insets insetCache;
    protected boolean leftToRightCache;
    protected Rectangle focusRect;
    protected Rectangle contentRect;
    protected Rectangle labelRect;
    protected Rectangle tickRect;
    protected Rectangle trackRect;
    protected Rectangle thumbRect;
    protected int trackBuffer;
    private transient boolean isDragging;
    protected BasicSliderUI.TrackListener trackListener;
    protected ChangeListener changeListener;
    protected ComponentListener componentListener;
    protected FocusListener focusListener;
    protected BasicSliderUI.ScrollListener scrollListener;
    protected PropertyChangeListener propertyChangeListener;
    private BasicSliderUI.Handler handler;
    private int lastValue;
    private Color shadowColor;
    private Color highlightColor;
    private Color focusColor;
    private boolean checkedLabelBaselines;
    private boolean sameLabelBaselines;
    private static Rectangle unionRect;
    
    protected Color getShadowColor() {
        return this.shadowColor;
    }
    
    protected Color getHighlightColor() {
        return this.highlightColor;
    }
    
    protected Color getFocusColor() {
        return this.focusColor;
    }
    
    protected boolean isDragging() {
        return this.isDragging;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicSliderUI((JSlider)component);
    }
    
    public BasicSliderUI(JSlider slider) {
        this.focusInsets = null;
        this.insetCache = null;
        this.leftToRightCache = true;
        this.focusRect = null;
        this.contentRect = null;
        this.labelRect = null;
        this.tickRect = null;
        this.trackRect = null;
        this.thumbRect = null;
        this.trackBuffer = 0;
    }
    
    @Override
    public void installUI(JComponent component) {
        this.slider = (JSlider)component;
        this.checkedLabelBaselines = false;
        this.slider.setEnabled(this.slider.isEnabled());
        LookAndFeel.installProperty(this.slider, "opaque", Boolean.TRUE);
        this.isDragging = false;
        this.trackListener = this.createTrackListener(this.slider);
        this.changeListener = this.createChangeListener(this.slider);
        this.componentListener = this.createComponentListener(this.slider);
        this.focusListener = this.createFocusListener(this.slider);
        this.scrollListener = this.createScrollListener(this.slider);
        this.propertyChangeListener = this.createPropertyChangeListener(this.slider);
        this.installDefaults(this.slider);
        this.installListeners(this.slider);
        this.installKeyboardActions(this.slider);
        (this.scrollTimer = new Timer(100, this.scrollListener)).setInitialDelay(300);
        this.insetCache = this.slider.getInsets();
        this.leftToRightCache = BasicGraphicsUtils.isLeftToRight(this.slider);
        this.focusRect = new Rectangle();
        this.contentRect = new Rectangle();
        this.labelRect = new Rectangle();
        this.tickRect = new Rectangle();
        this.trackRect = new Rectangle();
        this.thumbRect = new Rectangle();
        this.lastValue = this.slider.getValue();
        this.calculateGeometry();
    }
    
    @Override
    public void uninstallUI(JComponent obj) {
        if (obj != this.slider) {
            throw new IllegalComponentStateException(this + " was asked to deinstall() " + obj + " when it only knows about " + this.slider + ".");
        }
        this.scrollTimer.stop();
        this.scrollTimer = null;
        this.uninstallDefaults(this.slider);
        this.uninstallListeners(this.slider);
        this.uninstallKeyboardActions(this.slider);
        this.insetCache = null;
        this.leftToRightCache = true;
        this.focusRect = null;
        this.contentRect = null;
        this.labelRect = null;
        this.tickRect = null;
        this.trackRect = null;
        this.thumbRect = null;
        this.trackListener = null;
        this.changeListener = null;
        this.componentListener = null;
        this.focusListener = null;
        this.scrollListener = null;
        this.propertyChangeListener = null;
        this.slider = null;
        return;
    }
    
    protected void installDefaults(JSlider slider) {
        LookAndFeel.installBorder(slider, "Slider.border");
        LookAndFeel.installColorsAndFont(slider, "Slider.background", "Slider.foreground", "Slider.font");
        this.highlightColor = UIManager.getColor("Slider.highlight");
        this.shadowColor = UIManager.getColor("Slider.shadow");
        this.focusColor = UIManager.getColor("Slider.focus");
        this.focusInsets = (Insets)UIManager.get("Slider.focusInsets");
        if (this.focusInsets == null) {
            this.focusInsets = new InsetsUIResource(2, 2, 2, 2);
        }
    }
    
    protected void uninstallDefaults(JSlider c) {
        LookAndFeel.uninstallBorder(c);
        this.focusInsets = null;
    }
    
    protected BasicSliderUI.TrackListener createTrackListener(JSlider slider) {
        return new BasicSliderUI.TrackListener();
    }
    
    protected ChangeListener createChangeListener(JSlider slider) {
        return this.getHandler();
    }
    
    protected ComponentListener createComponentListener(JSlider slider) {
        return this.getHandler();
    }
    
    protected FocusListener createFocusListener(JSlider slider) {
        return this.getHandler();
    }
    
    protected BasicSliderUI.ScrollListener createScrollListener(JSlider slider) {
        return new BasicSliderUI.ScrollListener();
    }
    
    protected PropertyChangeListener createPropertyChangeListener(JSlider slider) {
        return this.getHandler();
    }
    
    private BasicSliderUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicSliderUI.Handler();
        }
        return this.handler;
    }
    
    protected void installListeners(JSlider slider) {
        slider.addMouseListener(this.trackListener);
        slider.addMouseMotionListener(this.trackListener);
        slider.addFocusListener(this.focusListener);
        slider.addComponentListener(this.componentListener);
        slider.addPropertyChangeListener(this.propertyChangeListener);
        slider.getModel().addChangeListener(this.changeListener);
    }
    
    protected void uninstallListeners(JSlider slider) {
        slider.removeMouseListener(this.trackListener);
        slider.removeMouseMotionListener(this.trackListener);
        slider.removeFocusListener(this.focusListener);
        slider.removeComponentListener(this.componentListener);
        slider.removePropertyChangeListener(this.propertyChangeListener);
        slider.getModel().removeChangeListener(this.changeListener);
        this.handler = null;
    }
    
    protected void installKeyboardActions(JSlider component) {
        SwingUtilities.replaceUIInputMap(component, 0, this.getInputMap(0, component));
        LazyActionMap.installLazyActionMap(component, BasicSliderUI.class, "Slider.actionMap");
    }
    
    InputMap getInputMap(int n, JSlider slider) {
        if (n == 0) {
            InputMap parent = (InputMap)DefaultLookup.get(slider, this, "Slider.focusInputMap");
            InputMap inputMap;
            if (slider.getComponentOrientation().isLeftToRight() || (inputMap = (InputMap)DefaultLookup.get(slider, this, "Slider.focusInputMap.RightToLeft")) == null) {
                return parent;
            }
            inputMap.setParent(parent);
            return inputMap;
        }
        else {
            return null;
        }
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicSliderUI.Actions("positiveUnitIncrement"));
        lazyActionMap.put(new BasicSliderUI.Actions("positiveBlockIncrement"));
        lazyActionMap.put(new BasicSliderUI.Actions("negativeUnitIncrement"));
        lazyActionMap.put(new BasicSliderUI.Actions("negativeBlockIncrement"));
        lazyActionMap.put(new BasicSliderUI.Actions("minScroll"));
        lazyActionMap.put(new BasicSliderUI.Actions("maxScroll"));
    }
    
    protected void uninstallKeyboardActions(JSlider slider) {
        SwingUtilities.replaceUIActionMap(slider, null);
        SwingUtilities.replaceUIInputMap(slider, 0, null);
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        if (this.slider.getPaintLabels() && this.labelsHaveSameBaselines()) {
            FontMetrics fontMetrics = this.slider.getFontMetrics(this.slider.getFont());
            Insets insets = this.slider.getInsets();
            Dimension thumbSize = this.getThumbSize();
            if (this.slider.getOrientation() == 0) {
                int tickLength = this.getTickLength();
                int n2 = height - insets.top - insets.bottom - this.focusInsets.top - this.focusInsets.bottom;
                int height2 = thumbSize.height;
                if (this.slider.getPaintTicks()) {
                    height2 += tickLength;
                }
                int n3 = insets.top + this.focusInsets.top + (n2 - (height2 + this.getHeightOfTallestLabel()) - 1) / 2;
                int n4 = n3 + height2;
                if (!this.slider.getPaintTicks()) {
                    tickLength = 0;
                }
                return n4 + tickLength + fontMetrics.getAscent();
            }
            else {
                Integer n = this.slider.getInverted() ? this.getLowestValue() : this.getHighestValue();
                if (n != null) {
                    int max = Math.max(fontMetrics.getHeight() / 2, thumbSize.height / 2);
                    return this.yPositionForValue((int)n, this.focusInsets.top + insets.top + max, height - this.focusInsets.top - this.focusInsets.bottom - insets.top - insets.bottom - max - max) - fontMetrics.getHeight() / 2 + fontMetrics.getAscent();
                }
            }
        }
        return 0;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        return Component.BaselineResizeBehavior.OTHER;
    }
    
    protected boolean labelsHaveSameBaselines() {
        if (!this.checkedLabelBaselines) {
            this.checkedLabelBaselines = true;
            Dictionary<Object, Object> labelTable = this.slider.getLabelTable();
            if (labelTable != null) {
                this.sameLabelBaselines = true;
                Enumeration<Object> elements = labelTable.elements();
                int n = -1;
                while (elements.hasMoreElements()) {
                    JComponent component = (JComponent)elements.nextElement();
                    Dimension preferredSize = component.getPreferredSize();
                    int baseline = component.getBaseline(preferredSize.width, preferredSize.height);
                    if (baseline >= 0) {
                        if (n == -1) {
                            n = baseline;
                        }
                        else {
                            if (n != baseline) {
                                this.sameLabelBaselines = false;
                                break;
                            }
                            continue;
                        }
                    }
                    else {
                        this.sameLabelBaselines = false;
                        break;
                    }
                }
            }
            else {
                this.sameLabelBaselines = false;
            }
        }
        return this.sameLabelBaselines;
    }
    
    public Dimension getPreferredHorizontalSize() {
        Dimension dimension = (Dimension)DefaultLookup.get(this.slider, this, "Slider.horizontalSize");
        if (dimension == null) {
            dimension = new Dimension(200, 21);
            return dimension;
        }
        return dimension;
    }
    
    public Dimension getPreferredVerticalSize() {
        Dimension dimension = (Dimension)DefaultLookup.get(this.slider, this, "Slider.verticalSize");
        if (dimension == null) {
            dimension = new Dimension(21, 200);
            return dimension;
        }
        return dimension;
    }
    
    public Dimension getMinimumHorizontalSize() {
        Dimension dimension = (Dimension)DefaultLookup.get(this.slider, this, "Slider.minimumHorizontalSize");
        if (dimension == null) {
            dimension = new Dimension(36, 21);
            return dimension;
        }
        return dimension;
    }
    
    public Dimension getMinimumVerticalSize() {
        Dimension dimension = (Dimension)DefaultLookup.get(this.slider, this, "Slider.minimumVerticalSize");
        if (dimension == null) {
            dimension = new Dimension(21, 36);
            return dimension;
        }
        return dimension;
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        this.recalculateIfInsetsChanged();
        if (this.slider.getOrientation() == 1) {
            Dimension dimension = new Dimension(this.getPreferredVerticalSize());
            dimension.width = this.insetCache.left + this.insetCache.right;
            dimension.width += this.focusInsets.left + this.focusInsets.right;
            dimension.width += this.trackRect.width + this.tickRect.width + this.labelRect.width;
            return dimension;
        }
        Dimension dimension = new Dimension(this.getPreferredHorizontalSize());
        dimension.height = this.insetCache.top + this.insetCache.bottom;
        dimension.height += this.focusInsets.top + this.focusInsets.bottom;
        dimension.height += this.trackRect.height + this.tickRect.height + this.labelRect.height;
        return dimension;
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        this.recalculateIfInsetsChanged();
        if (this.slider.getOrientation() == 1) {
            Dimension dimension = new Dimension(this.getMinimumVerticalSize());
            dimension.width = this.insetCache.left + this.insetCache.right;
            dimension.width += this.focusInsets.left + this.focusInsets.right;
            dimension.width += this.trackRect.width + this.tickRect.width + this.labelRect.width;
            return dimension;
        }
        Dimension dimension = new Dimension(this.getMinimumHorizontalSize());
        dimension.height = this.insetCache.top + this.insetCache.bottom;
        dimension.height += this.focusInsets.top + this.focusInsets.bottom;
        dimension.height += this.trackRect.height + this.tickRect.height + this.labelRect.height;
        return dimension;
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        if (this.slider.getOrientation() == 1) {
            preferredSize.height = 32767;
            return preferredSize;
        }
        preferredSize.width = 32767;
        return preferredSize;
    }
    
    protected void calculateGeometry() {
        this.calculateFocusRect();
        this.calculateContentRect();
        this.calculateThumbSize();
        this.calculateTrackBuffer();
        this.calculateTrackRect();
        this.calculateTickRect();
        this.calculateLabelRect();
        this.calculateThumbLocation();
    }
    
    protected void calculateFocusRect() {
        this.focusRect.x = this.insetCache.left;
        this.focusRect.y = this.insetCache.top;
        this.focusRect.width = this.slider.getWidth() - (this.insetCache.left + this.insetCache.right);
        this.focusRect.height = this.slider.getHeight() - (this.insetCache.top + this.insetCache.bottom);
    }
    
    protected void calculateThumbSize() {
        Dimension thumbSize = this.getThumbSize();
        this.thumbRect.setSize(thumbSize.width, thumbSize.height);
    }
    
    protected void calculateContentRect() {
        this.contentRect.x = this.focusRect.x + this.focusInsets.left;
        this.contentRect.y = this.focusRect.y + this.focusInsets.top;
        this.contentRect.width = this.focusRect.width - (this.focusInsets.left + this.focusInsets.right);
        this.contentRect.height = this.focusRect.height - (this.focusInsets.top + this.focusInsets.bottom);
    }
    
    private int getTickSpacing() {
        int majorTickSpacing = this.slider.getMajorTickSpacing();
        int minorTickSpacing = this.slider.getMinorTickSpacing();
        if (minorTickSpacing > 0) {
        }
        else if (majorTickSpacing > 0) {
            minorTickSpacing = majorTickSpacing;
        }
        else {
            minorTickSpacing = 0;
        }
        return minorTickSpacing;
    }
    
    protected void calculateThumbLocation() {
        if (this.slider.getSnapToTicks()) {
            int value = this.slider.getValue();
            int tickSpacing = this.getTickSpacing();
            if (tickSpacing != 0) {
                if ((value - this.slider.getMinimum()) % tickSpacing != 0) {
                    float a = (float)(value - this.slider.getMinimum()) / (float)tickSpacing;
                    int round = Math.round(a);
                    if ((double)(a - (float)(int)a) == 0.5 && value < this.lastValue) {
                        --round;
                    }
                    value = this.slider.getMinimum() + round * tickSpacing;
                }
                if (value != value) {
                    this.slider.setValue(value);
                }
            }
        }
        if (this.slider.getOrientation() == 0) {
            this.thumbRect.x = this.xPositionForValue(this.slider.getValue()) - this.thumbRect.width / 2;
            this.thumbRect.y = this.trackRect.y;
        }
        else {
            int yPositionForValue = this.yPositionForValue(this.slider.getValue());
            this.thumbRect.x = this.trackRect.x;
            this.thumbRect.y = yPositionForValue - this.thumbRect.height / 2;
        }
    }
    
    protected void calculateTrackBuffer() {
        if (this.slider.getPaintLabels() && this.slider.getLabelTable() != null) {
            Component highestValueLabel = this.getHighestValueLabel();
            Component lowestValueLabel = this.getLowestValueLabel();
            if (this.slider.getOrientation() == 0) {
                this.trackBuffer = Math.max(highestValueLabel.getBounds().width, lowestValueLabel.getBounds().width) / 2;
                this.trackBuffer = Math.max(this.trackBuffer, this.thumbRect.width / 2);
            }
            else {
                this.trackBuffer = Math.max(highestValueLabel.getBounds().height, lowestValueLabel.getBounds().height) / 2;
                this.trackBuffer = Math.max(this.trackBuffer, this.thumbRect.height / 2);
            }
        }
        else if (this.slider.getOrientation() == 0) {
            this.trackBuffer = this.thumbRect.width / 2;
        }
        else {
            this.trackBuffer = this.thumbRect.height / 2;
        }
    }
    
    protected void calculateTrackRect() {
        if (this.slider.getOrientation() == 0) {
            int height = this.thumbRect.height;
            if (this.slider.getPaintTicks()) {
                height += this.getTickLength();
            }
            if (this.slider.getPaintLabels()) {
                height += this.getHeightOfTallestLabel();
            }
            this.trackRect.x = this.contentRect.x + this.trackBuffer;
            this.trackRect.y = this.contentRect.y + (this.contentRect.height - height - 1) / 2;
            this.trackRect.width = this.contentRect.width - this.trackBuffer * 2;
            this.trackRect.height = this.thumbRect.height;
        }
        else {
            int width = this.thumbRect.width;
            if (BasicGraphicsUtils.isLeftToRight(this.slider)) {
                if (this.slider.getPaintTicks()) {
                    width += this.getTickLength();
                }
                if (this.slider.getPaintLabels()) {
                    width += this.getWidthOfWidestLabel();
                }
            }
            else {
                if (this.slider.getPaintTicks()) {
                    width -= this.getTickLength();
                }
                if (this.slider.getPaintLabels()) {
                    width -= this.getWidthOfWidestLabel();
                }
            }
            this.trackRect.x = this.contentRect.x + (this.contentRect.width - width - 1) / 2;
            this.trackRect.y = this.contentRect.y + this.trackBuffer;
            this.trackRect.width = this.thumbRect.width;
            this.trackRect.height = this.contentRect.height - this.trackBuffer * 2;
        }
    }
    
    protected int getTickLength() {
        return 8;
    }
    
    protected void calculateTickRect() {
        if (this.slider.getOrientation() == 0) {
            this.tickRect.x = this.trackRect.x;
            this.tickRect.y = this.trackRect.y + this.trackRect.height;
            this.tickRect.width = this.trackRect.width;
            this.tickRect.height = (this.slider.getPaintTicks() ? this.getTickLength() : 0);
        }
        else {
            this.tickRect.width = (this.slider.getPaintTicks() ? this.getTickLength() : 0);
            if (BasicGraphicsUtils.isLeftToRight(this.slider)) {
                this.tickRect.x = this.trackRect.x + this.trackRect.width;
            }
            else {
                this.tickRect.x = this.trackRect.x - this.tickRect.width;
            }
            this.tickRect.y = this.trackRect.y;
            this.tickRect.height = this.trackRect.height;
        }
    }
    
    protected void calculateLabelRect() {
        if (this.slider.getPaintLabels()) {
            if (this.slider.getOrientation() == 0) {
                this.labelRect.x = this.tickRect.x - this.trackBuffer;
                this.labelRect.y = this.tickRect.y + this.tickRect.height;
                this.labelRect.width = this.tickRect.width + this.trackBuffer * 2;
                this.labelRect.height = this.getHeightOfTallestLabel();
            }
            else {
                if (BasicGraphicsUtils.isLeftToRight(this.slider)) {
                    this.labelRect.x = this.tickRect.x + this.tickRect.width;
                    this.labelRect.width = this.getWidthOfWidestLabel();
                }
                else {
                    this.labelRect.width = this.getWidthOfWidestLabel();
                    this.labelRect.x = this.tickRect.x - this.labelRect.width;
                }
                this.labelRect.y = this.tickRect.y - this.trackBuffer;
                this.labelRect.height = this.tickRect.height + this.trackBuffer * 2;
            }
        }
        else if (this.slider.getOrientation() == 0) {
            this.labelRect.x = this.tickRect.x;
            this.labelRect.y = this.tickRect.y + this.tickRect.height;
            this.labelRect.width = this.tickRect.width;
            this.labelRect.height = 0;
        }
        else {
            if (BasicGraphicsUtils.isLeftToRight(this.slider)) {
                this.labelRect.x = this.tickRect.x + this.tickRect.width;
            }
            else {
                this.labelRect.x = this.tickRect.x;
            }
            this.labelRect.y = this.tickRect.y;
            this.labelRect.width = 0;
            this.labelRect.height = this.tickRect.height;
        }
    }
    
    protected Dimension getThumbSize() {
        Dimension dimension = new Dimension();
        if (this.slider.getOrientation() == 1) {
            dimension.width = 20;
            dimension.height = 11;
            return dimension;
        }
        dimension.width = 11;
        dimension.height = 20;
        return dimension;
    }
    
    protected int getWidthOfWidestLabel() {
        Dictionary<Object, Object> labelTable = this.slider.getLabelTable();
        int max = 0;
        if (labelTable != null) {
            Enumeration<Object> keys = labelTable.keys();
            while (keys.hasMoreElements()) {
                max = Math.max(((JComponent)labelTable.get(keys.nextElement())).getPreferredSize().width, max);
            }
        }
        return max;
    }
    
    protected int getHeightOfTallestLabel() {
        Dictionary<Object, Object> labelTable = this.slider.getLabelTable();
        int max = 0;
        if (labelTable != null) {
            Enumeration<Object> keys = labelTable.keys();
            while (keys.hasMoreElements()) {
                max = Math.max(((JComponent)labelTable.get(keys.nextElement())).getPreferredSize().height, max);
            }
        }
        return max;
    }
    
    protected int getWidthOfHighValueLabel() {
        Component highestValueLabel = this.getHighestValueLabel();
        int width = 0;
        if (highestValueLabel != null) {
            width = highestValueLabel.getPreferredSize().width;
        }
        return width;
    }
    
    protected int getWidthOfLowValueLabel() {
        Component lowestValueLabel = this.getLowestValueLabel();
        int width = 0;
        if (lowestValueLabel != null) {
            width = lowestValueLabel.getPreferredSize().width;
        }
        return width;
    }
    
    protected int getHeightOfHighValueLabel() {
        Component highestValueLabel = this.getHighestValueLabel();
        int height = 0;
        if (highestValueLabel != null) {
            height = highestValueLabel.getPreferredSize().height;
        }
        return height;
    }
    
    protected int getHeightOfLowValueLabel() {
        Component lowestValueLabel = this.getLowestValueLabel();
        int height = 0;
        if (lowestValueLabel != null) {
            height = lowestValueLabel.getPreferredSize().height;
        }
        return height;
    }
    
    protected boolean drawInverted() {
        if (this.slider.getOrientation() == 0) {
            if (BasicGraphicsUtils.isLeftToRight(this.slider)) {
                return this.slider.getInverted();
            }
            return !this.slider.getInverted();
        }
        else {
            return this.slider.getInverted();
        }
    }
    
    protected Integer getHighestValue() {
        Dictionary<Object, Object> labelTable = this.slider.getLabelTable();
        if (labelTable == null) {
            return null;
        }
        Enumeration<Object> keys = labelTable.keys();
        Integer n = null;
        while (keys.hasMoreElements()) {
            Integer n2 = (Integer)keys.nextElement();
            if (n == null || (int)n2 > (int)n) {
                n = n2;
            }
            else {
                continue;
            }
        }
        return n;
    }
    
    protected Integer getLowestValue() {
        Dictionary<Object, Object> labelTable = this.slider.getLabelTable();
        if (labelTable == null) {
            return null;
        }
        Enumeration<Object> keys = labelTable.keys();
        Integer n = null;
        while (keys.hasMoreElements()) {
            Integer n2 = (Integer)keys.nextElement();
            if (n == null || (int)n2 < (int)n) {
                n = n2;
            }
            else {
                continue;
            }
        }
        return n;
    }
    
    protected Component getLowestValueLabel() {
        Integer lowestValue = this.getLowestValue();
        if (lowestValue != null) {
            return (Component)this.slider.getLabelTable().get(lowestValue);
        }
        return null;
    }
    
    protected Component getHighestValueLabel() {
        Integer highestValue = this.getHighestValue();
        if (highestValue != null) {
            return (Component)this.slider.getLabelTable().get(highestValue);
        }
        return null;
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        this.recalculateIfInsetsChanged();
        this.recalculateIfOrientationChanged();
        Rectangle clipBounds = graphics.getClipBounds();
        if (!clipBounds.intersects(this.trackRect) && this.slider.getPaintTrack()) {
            this.calculateGeometry();
        }
        if (this.slider.getPaintTrack() && clipBounds.intersects(this.trackRect)) {
            this.paintTrack(graphics);
        }
        if (this.slider.getPaintTicks() && clipBounds.intersects(this.tickRect)) {
            this.paintTicks(graphics);
        }
        if (this.slider.getPaintLabels() && clipBounds.intersects(this.labelRect)) {
            this.paintLabels(graphics);
        }
        if (this.slider.hasFocus() && clipBounds.intersects(this.focusRect)) {
            this.paintFocus(graphics);
        }
        if (clipBounds.intersects(this.thumbRect)) {
            this.paintThumb(graphics);
        }
    }
    
    protected void recalculateIfInsetsChanged() {
        Insets insets = this.slider.getInsets();
        if (!insets.equals(this.insetCache)) {
            this.insetCache = insets;
            this.calculateGeometry();
        }
    }
    
    protected void recalculateIfOrientationChanged() {
        int leftToRight = BasicGraphicsUtils.isLeftToRight(this.slider);
        if (leftToRight != (this.leftToRightCache ? 1 : 0)) {
            this.leftToRightCache = leftToRight;
            this.calculateGeometry();
        }
    }
    
    public void paintFocus(Graphics graphics) {
        graphics.setColor(this.getFocusColor());
        BasicGraphicsUtils.drawDashedRect(graphics, this.focusRect.x, this.focusRect.y, this.focusRect.width, this.focusRect.height);
    }
    
    public void paintTrack(Graphics graphics) {
        Rectangle trackRect = this.trackRect;
        if (this.slider.getOrientation() == 0) {
            int n2 = trackRect.height / 2 - 2;
            int width = trackRect.width;
            graphics.translate(trackRect.x, trackRect.y + n2);
            graphics.setColor(this.getShadowColor());
            graphics.drawLine(0, 0, width - 1, 0);
            graphics.drawLine(0, 1, 0, 2);
            graphics.setColor(this.getHighlightColor());
            graphics.drawLine(0, 3, width, 3);
            graphics.drawLine(width, 0, width, 3);
            graphics.setColor(Color.black);
            graphics.drawLine(1, 1, width - 2, 1);
            graphics.translate(-trackRect.x, -(trackRect.y + n2));
        }
        else {
            int n = trackRect.width / 2 - 2;
            int height = trackRect.height;
            graphics.translate(trackRect.x + n, trackRect.y);
            graphics.setColor(this.getShadowColor());
            graphics.drawLine(0, 0, 0, height - 1);
            graphics.drawLine(1, 0, 2, 0);
            graphics.setColor(this.getHighlightColor());
            graphics.drawLine(3, 0, 3, height);
            graphics.drawLine(0, height, 3, height);
            graphics.setColor(Color.black);
            graphics.drawLine(1, 1, 1, height - 2);
            graphics.translate(-(trackRect.x + n), -trackRect.y);
        }
    }
    
    public void paintTicks(Graphics graphics) {
        Rectangle tickRect = this.tickRect;
        graphics.setColor(DefaultLookup.getColor(this.slider, this, "Slider.tickColor", Color.black));
        if (this.slider.getOrientation() == 0) {
            graphics.translate(0, tickRect.y);
            if (this.slider.getMinorTickSpacing() > 0) {
                int k = this.slider.getMinimum();
                while (k <= this.slider.getMaximum()) {
                    this.paintMinorTickForHorizSlider(graphics, tickRect, this.xPositionForValue(k));
                    if (Integer.MAX_VALUE - this.slider.getMinorTickSpacing() < k) {
                        break;
                    }
                    k += this.slider.getMinorTickSpacing();
                }
            }
            if (this.slider.getMajorTickSpacing() > 0) {
                int l = this.slider.getMinimum();
                while (l <= this.slider.getMaximum()) {
                    this.paintMajorTickForHorizSlider(graphics, tickRect, this.xPositionForValue(l));
                    if (Integer.MAX_VALUE - this.slider.getMajorTickSpacing() < l) {
                        break;
                    }
                    l += this.slider.getMajorTickSpacing();
                }
            }
            graphics.translate(0, -tickRect.y);
        }
        else {
            graphics.translate(tickRect.x, 0);
            if (this.slider.getMinorTickSpacing() > 0) {
                int n = 0;
                if (!BasicGraphicsUtils.isLeftToRight(this.slider)) {
                    n = tickRect.width - tickRect.width / 2;
                    graphics.translate(n, 0);
                }
                int i = this.slider.getMinimum();
                while (i <= this.slider.getMaximum()) {
                    this.paintMinorTickForVertSlider(graphics, tickRect, this.yPositionForValue(i));
                    if (Integer.MAX_VALUE - this.slider.getMinorTickSpacing() < i) {
                        break;
                    }
                    i += this.slider.getMinorTickSpacing();
                }
                if (!BasicGraphicsUtils.isLeftToRight(this.slider)) {
                    graphics.translate(-n, 0);
                }
            }
            if (this.slider.getMajorTickSpacing() > 0) {
                if (!BasicGraphicsUtils.isLeftToRight(this.slider)) {
                    graphics.translate(2, 0);
                }
                int j = this.slider.getMinimum();
                while (j <= this.slider.getMaximum()) {
                    this.paintMajorTickForVertSlider(graphics, tickRect, this.yPositionForValue(j));
                    if (Integer.MAX_VALUE - this.slider.getMajorTickSpacing() < j) {
                        break;
                    }
                    j += this.slider.getMajorTickSpacing();
                }
                if (!BasicGraphicsUtils.isLeftToRight(this.slider)) {
                    graphics.translate(-2, 0);
                }
            }
            graphics.translate(-tickRect.x, 0);
        }
    }
    
    protected void paintMinorTickForHorizSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.drawLine(n, 0, n, rectangle.height / 2 - 1);
    }
    
    protected void paintMajorTickForHorizSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.drawLine(n, 0, n, rectangle.height - 2);
    }
    
    protected void paintMinorTickForVertSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.drawLine(0, n, rectangle.width / 2 - 1, n);
    }
    
    protected void paintMajorTickForVertSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.drawLine(0, n, rectangle.width - 2, n);
    }
    
    public void paintLabels(Graphics graphics) {
        Rectangle labelRect = this.labelRect;
        Dictionary<Object, Object> labelTable = this.slider.getLabelTable();
        if (labelTable != null) {
            Enumeration<Object> keys = labelTable.keys();
            int minimum = this.slider.getMinimum();
            int maximum = this.slider.getMaximum();
            int enabled = this.slider.isEnabled();
            while (keys.hasMoreElements()) {
                Integer n = (Integer)keys.nextElement();
                int intValue = (int)n;
                if (intValue >= minimum && intValue <= maximum) {
                    JComponent component = (JComponent)labelTable.get(n);
                    component.setEnabled((boolean)enabled);
                    if ((component instanceof JLabel)) {
                        Icon icon = component.isEnabled() ? ((JLabel)component).getIcon() : ((JLabel)component).getDisabledIcon();
                        if ((icon instanceof ImageIcon)) {
                            Toolkit.getDefaultToolkit().checkImage(((ImageIcon)icon).getImage(), -1, -1, this.slider);
                        }
                    }
                    if (this.slider.getOrientation() == 0) {
                        graphics.translate(0, labelRect.y);
                        this.paintHorizontalLabel(graphics, intValue, component);
                        graphics.translate(0, -labelRect.y);
                    }
                    else {
                        int n2 = 0;
                        if (!BasicGraphicsUtils.isLeftToRight(this.slider)) {
                            n2 = labelRect.width - component.getPreferredSize().width;
                        }
                        graphics.translate(labelRect.x + n2, 0);
                        this.paintVerticalLabel(graphics, intValue, component);
                        graphics.translate(-labelRect.x - n2, 0);
                    }
                }
                else {
                    continue;
                }
            }
        }
    }
    
    protected void paintHorizontalLabel(Graphics g, int n, Component component) {
        int n2 = this.xPositionForValue(n) - component.getPreferredSize().width / 2;
        g.translate(n2, 0);
        component.paint(g);
        g.translate(-n2, 0);
    }
    
    protected void paintVerticalLabel(Graphics g, int n, Component component) {
        int n2 = this.yPositionForValue(n) - component.getPreferredSize().height / 2;
        g.translate(0, n2);
        component.paint(g);
        g.translate(0, -n2);
    }
    
    public void paintThumb(Graphics graphics) {
        Rectangle thumbRect = this.thumbRect;
        int width = thumbRect.width;
        int height = thumbRect.height;
        graphics.translate(thumbRect.x, thumbRect.y);
        if (this.slider.isEnabled()) {
            graphics.setColor(this.slider.getBackground());
        }
        else {
            graphics.setColor(this.slider.getBackground().darker());
        }
        Boolean b = (Boolean)this.slider.getClientProperty("Slider.paintThumbArrowShape");
        if ((!this.slider.getPaintTicks() && b == null) || b == Boolean.FALSE) {
            graphics.fillRect(0, 0, width, height);
            graphics.setColor(Color.black);
            graphics.drawLine(0, height - 1, width - 1, height - 1);
            graphics.drawLine(width - 1, 0, width - 1, height - 1);
            graphics.setColor(this.highlightColor);
            graphics.drawLine(0, 0, 0, height - 2);
            graphics.drawLine(1, 0, width - 2, 0);
            graphics.setColor(this.shadowColor);
            graphics.drawLine(1, height - 2, width - 2, height - 2);
            graphics.drawLine(width - 2, 1, width - 2, height - 3);
        }
        else if (this.slider.getOrientation() == 0) {
            int n2 = width / 2;
            graphics.fillRect(1, 1, width - 3, height - 1 - n2);
            Polygon p3 = new Polygon();
            p3.addPoint(1, height - n2);
            p3.addPoint(n2 - 1, height - 1);
            p3.addPoint(width - 2, height - 1 - n2);
            graphics.fillPolygon(p3);
            graphics.setColor(this.highlightColor);
            graphics.drawLine(0, 0, width - 2, 0);
            graphics.drawLine(0, 1, 0, height - 1 - n2);
            graphics.drawLine(0, height - n2, n2 - 1, height - 1);
            graphics.setColor(Color.black);
            graphics.drawLine(width - 1, 0, width - 1, height - 2 - n2);
            graphics.drawLine(width - 1, height - 1 - n2, width - 1 - n2, height - 1);
            graphics.setColor(this.shadowColor);
            graphics.drawLine(width - 2, 1, width - 2, height - 2 - n2);
            graphics.drawLine(width - 2, height - 1 - n2, width - 1 - n2, height - 2);
        }
        else {
            int n = height / 2;
            if (BasicGraphicsUtils.isLeftToRight(this.slider)) {
                graphics.fillRect(1, 1, width - 1 - n, height - 3);
                Polygon p2 = new Polygon();
                p2.addPoint(width - n - 1, 0);
                p2.addPoint(width - 1, n);
                p2.addPoint(width - 1 - n, height - 2);
                graphics.fillPolygon(p2);
                graphics.setColor(this.highlightColor);
                graphics.drawLine(0, 0, 0, height - 2);
                graphics.drawLine(1, 0, width - 1 - n, 0);
                graphics.drawLine(width - n - 1, 0, width - 1, n);
                graphics.setColor(Color.black);
                graphics.drawLine(0, height - 1, width - 2 - n, height - 1);
                graphics.drawLine(width - 1 - n, height - 1, width - 1, height - 1 - n);
                graphics.setColor(this.shadowColor);
                graphics.drawLine(1, height - 2, width - 2 - n, height - 2);
                graphics.drawLine(width - 1 - n, height - 2, width - 2, height - n - 1);
            }
            else {
                graphics.fillRect(5, 1, width - 1 - n, height - 3);
                Polygon p = new Polygon();
                p.addPoint(n, 0);
                p.addPoint(0, n);
                p.addPoint(n, height - 2);
                graphics.fillPolygon(p);
                graphics.setColor(this.highlightColor);
                graphics.drawLine(n - 1, 0, width - 2, 0);
                graphics.drawLine(0, n, n, 0);
                graphics.setColor(Color.black);
                graphics.drawLine(0, height - 1 - n, n, height - 1);
                graphics.drawLine(n, height - 1, width - 1, height - 1);
                graphics.setColor(this.shadowColor);
                graphics.drawLine(n, height - 2, width - 2, height - 2);
                graphics.drawLine(width - 1, 1, width - 1, height - 2);
            }
        }
        graphics.translate(-thumbRect.x, -thumbRect.y);
    }
    
    public void setThumbLocation(int x, int y) {
        BasicSliderUI.unionRect.setBounds(this.thumbRect);
        this.thumbRect.setLocation(x, y);
        SwingUtilities.computeUnion(this.thumbRect.x, this.thumbRect.y, this.thumbRect.width, this.thumbRect.height, BasicSliderUI.unionRect);
        this.slider.repaint(BasicSliderUI.unionRect.x, BasicSliderUI.unionRect.y, BasicSliderUI.unionRect.width, BasicSliderUI.unionRect.height);
    }
    
    public void scrollByBlock(int n) {
        synchronized (this.slider) {
            int n2 = (this.slider.getMaximum() - this.slider.getMinimum()) / 10;
            if (n2 == 0) {
                n2 = 1;
            }
            if (this.slider.getSnapToTicks()) {
                int tickSpacing = this.getTickSpacing();
                if (n2 < tickSpacing) {
                    n2 = tickSpacing;
                }
            }
            this.slider.setValue(this.slider.getValue() + n2 * ((n > 0) ? 1 : -1));
        }
    }
    
    public void scrollByUnit(int n) {
        synchronized (this.slider) {
            int n2 = (n > 0) ? 1 : -1;
            if (this.slider.getSnapToTicks()) {
                n2 *= this.getTickSpacing();
            }
            this.slider.setValue(this.slider.getValue() + n2);
        }
    }
    
    protected void scrollDueToClickInTrack(int n) {
        this.scrollByBlock(n);
    }
    
    protected int xPositionForValue(int n) {
        int minimum = this.slider.getMinimum();
        double n2 = (double)this.trackRect.width / ((double)this.slider.getMaximum() - (double)minimum);
        int x = this.trackRect.x;
        int a = this.trackRect.x + (this.trackRect.width - 1);
        int b;
        if (!this.drawInverted()) {
            b = (int)((long)x + Math.round(n2 * ((double)n - (double)minimum)));
        }
        else {
            b = (int)((long)a - Math.round(n2 * ((double)n - (double)minimum)));
        }
        return Math.min(a, Math.max(x, b));
    }
    
    protected int yPositionForValue(int n) {
        return this.yPositionForValue(n, this.trackRect.y, this.trackRect.height);
    }
    
    protected int yPositionForValue(int n, int a, int n2) {
        int minimum = this.slider.getMinimum();
        int maximum = this.slider.getMaximum();
        double n3 = (double)n2 / ((double)maximum - (double)minimum);
        int a2 = a + (n2 - 1);
        int b;
        if (!this.drawInverted()) {
            b = (int)((long)a + Math.round(n3 * ((double)maximum - (double)n)));
        }
        else {
            b = (int)((long)a + Math.round(n3 * ((double)n - (double)minimum)));
        }
        return Math.min(a2, Math.max(a, b));
    }
    
    public int valueForYPosition(int n) {
        int minimum = this.slider.getMinimum();
        int maximum = this.slider.getMaximum();
        int height = this.trackRect.height;
        int y = this.trackRect.y;
        int n2 = this.trackRect.y + (this.trackRect.height - 1);
        if (n <= y) {
            maximum = (this.drawInverted() ? minimum : maximum);
        }
        else if (n >= n2) {
            maximum = (this.drawInverted() ? maximum : minimum);
        }
        else {
            int n3 = (int)Math.round((double)(n - y) * (((double)maximum - (double)minimum) / (double)height));
            maximum = (this.drawInverted() ? (minimum + n3) : (maximum - n3));
        }
        return maximum;
    }
    
    public int valueForXPosition(int n) {
        int minimum = this.slider.getMinimum();
        int maximum = this.slider.getMaximum();
        int width = this.trackRect.width;
        int x = this.trackRect.x;
        int n2 = this.trackRect.x + (this.trackRect.width - 1);
        if (n <= x) {
            minimum = (this.drawInverted() ? maximum : minimum);
        }
        else if (n >= n2) {
            minimum = (this.drawInverted() ? minimum : maximum);
        }
        else {
            int n3 = (int)Math.round((double)(n - x) * (((double)maximum - (double)minimum) / (double)width));
            minimum = (this.drawInverted() ? (maximum - n3) : (minimum + n3));
        }
        return minimum;
    }
    
    static BasicSliderUI.Handler access$100(BasicSliderUI basicSliderUI) {
        return basicSliderUI.getHandler();
    }
    
    static {
        SHARED_ACTION = new BasicSliderUI.Actions();
        BasicSliderUI.unionRect = new Rectangle();
    }
    
    public class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            BasicSliderUI.access$100(BasicSliderUI.this).propertyChange(propertyChangeEvent);
        }
    }
    
    private class Handler implements ChangeListener, ComponentListener, FocusListener, PropertyChangeListener
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            if (!BasicSliderUI.this.isDragging) {
                BasicSliderUI.this.calculateThumbLocation();
                BasicSliderUI.this.slider.repaint();
            }
            BasicSliderUI.this.lastValue = BasicSliderUI.this.slider.getValue();
        }
        
        @Override
        public void componentHidden(ComponentEvent componentEvent) {
        }
        
        @Override
        public void componentMoved(ComponentEvent componentEvent) {
        }
        
        @Override
        public void componentResized(ComponentEvent componentEvent) {
            BasicSliderUI.this.calculateGeometry();
            BasicSliderUI.this.slider.repaint();
        }
        
        @Override
        public void componentShown(ComponentEvent componentEvent) {
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicSliderUI.this.slider.repaint();
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            BasicSliderUI.this.slider.repaint();
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if (propertyName == "orientation" || propertyName == "inverted" || propertyName == "labelTable" || propertyName == "majorTickSpacing" || propertyName == "minorTickSpacing" || propertyName == "paintTicks" || propertyName == "paintTrack" || propertyName == "font" || propertyName == "paintLabels" || propertyName == "Slider.paintThumbArrowShape") {
                BasicSliderUI.this.checkedLabelBaselines = false;
                BasicSliderUI.this.calculateGeometry();
                BasicSliderUI.this.slider.repaint();
            }
            else if (propertyName == "componentOrientation") {
                BasicSliderUI.this.calculateGeometry();
                BasicSliderUI.this.slider.repaint();
                SwingUtilities.replaceUIInputMap(BasicSliderUI.this.slider, 0, BasicSliderUI.this.getInputMap(0, BasicSliderUI.this.slider));
            }
            else if (propertyName == "model") {
                ((BoundedRangeModel)propertyChangeEvent.getOldValue()).removeChangeListener(BasicSliderUI.this.changeListener);
                ((BoundedRangeModel)propertyChangeEvent.getNewValue()).addChangeListener(BasicSliderUI.this.changeListener);
                BasicSliderUI.this.calculateThumbLocation();
                BasicSliderUI.this.slider.repaint();
            }
        }
    }
    
    public class ChangeHandler implements ChangeListener
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            BasicSliderUI.access$100(BasicSliderUI.this).stateChanged(changeEvent);
        }
    }
    
    public class TrackListener extends MouseInputAdapter
    {
        protected transient int offset;
        protected transient int currentMouseX;
        protected transient int currentMouseY;
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            if (!BasicSliderUI.this.slider.isEnabled()) {
                return;
            }
            this.offset = 0;
            BasicSliderUI.this.scrollTimer.stop();
            BasicSliderUI.this.isDragging = false;
            BasicSliderUI.this.slider.setValueIsAdjusting(false);
            BasicSliderUI.this.slider.repaint();
            return;
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            if (!BasicSliderUI.this.slider.isEnabled()) {
                return;
            }
            BasicSliderUI.this.calculateGeometry();
            this.currentMouseX = mouseEvent.getX();
            this.currentMouseY = mouseEvent.getY();
            if (BasicSliderUI.this.slider.isRequestFocusEnabled()) {
                BasicSliderUI.this.slider.requestFocus();
            }
            if (BasicSliderUI.this.thumbRect.contains(this.currentMouseX, this.currentMouseY)) {
                if (UIManager.getBoolean("Slider.onlyLeftMouseButtonDrag") && !SwingUtilities.isLeftMouseButton(mouseEvent)) {
                    return;
                }
                switch (BasicSliderUI.this.slider.getOrientation()) {
                    case 1: {
                        this.offset = this.currentMouseY - BasicSliderUI.this.thumbRect.y;
                        break;
                    }
                    case 0: {
                        this.offset = this.currentMouseX - BasicSliderUI.this.thumbRect.x;
                        break;
                    }
                }
                BasicSliderUI.this.isDragging = true;
                return;
            }
            else {
                if (!SwingUtilities.isLeftMouseButton(mouseEvent)) {
                    return;
                }
                BasicSliderUI.this.isDragging = false;
                BasicSliderUI.this.slider.setValueIsAdjusting(true);
                Dimension size = BasicSliderUI.this.slider.getSize();
                int direction = 1;
                switch (BasicSliderUI.this.slider.getOrientation()) {
                    case 1: {
                        if (BasicSliderUI.this.thumbRect.isEmpty()) {
                            int n = size.height / 2;
                            if (!BasicSliderUI.this.drawInverted()) {
                                direction = ((this.currentMouseY < n) ? 1 : -1);
                            }
                            else {
                                direction = ((this.currentMouseY < n) ? -1 : 1);
                            }
                            break;
                        }
                        else {
                            int y = BasicSliderUI.this.thumbRect.y;
                            if (!BasicSliderUI.this.drawInverted()) {
                                direction = ((this.currentMouseY < y) ? 1 : -1);
                            }
                            else {
                                direction = ((this.currentMouseY < y) ? -1 : 1);
                            }
                            break;
                        }
                    }
                    case 0: {
                        if (BasicSliderUI.this.thumbRect.isEmpty()) {
                            int n2 = size.width / 2;
                            if (!BasicSliderUI.this.drawInverted()) {
                                direction = ((this.currentMouseX < n2) ? -1 : 1);
                            }
                            else {
                                direction = ((this.currentMouseX < n2) ? 1 : -1);
                            }
                            break;
                        }
                        else {
                            int x = BasicSliderUI.this.thumbRect.x;
                            if (!BasicSliderUI.this.drawInverted()) {
                                direction = ((this.currentMouseX < x) ? -1 : 1);
                                break;
                            }
                            direction = ((this.currentMouseX < x) ? 1 : -1);
                            break;
                        }
                    }
                }
                if (this.shouldScroll(direction)) {
                    BasicSliderUI.this.scrollDueToClickInTrack(direction);
                }
                if (this.shouldScroll(direction)) {
                    BasicSliderUI.this.scrollTimer.stop();
                    BasicSliderUI.this.scrollListener.setDirection(direction);
                    BasicSliderUI.this.scrollTimer.start();
                }
                return;
            }
        }
        
        public boolean shouldScroll(int n) {
            Rectangle thumbRect = BasicSliderUI.this.thumbRect;
            if (BasicSliderUI.this.slider.getOrientation() == 1) {
                if (!(BasicSliderUI.this.drawInverted() ? (n >= 0) : (n <= 0))) {
                    if (thumbRect.y <= this.currentMouseY) {
                        return false;
                    }
                }
                else if (thumbRect.y + thumbRect.height >= this.currentMouseY) {
                    return false;
                }
            }
            else if (!(BasicSliderUI.this.drawInverted() ? (n >= 0) : (n <= 0))) {
                if (thumbRect.x + thumbRect.width >= this.currentMouseX) {
                    return false;
                }
            }
            else if (thumbRect.x <= this.currentMouseX) {
                return false;
            }
            return (n <= 0 || BasicSliderUI.this.slider.getValue() + BasicSliderUI.this.slider.getExtent() < BasicSliderUI.this.slider.getMaximum()) && (n >= 0 || BasicSliderUI.this.slider.getValue() > BasicSliderUI.this.slider.getMinimum());
        }
        
        @Override
        public void mouseDragged(MouseEvent mouseEvent) {
            if (!BasicSliderUI.this.slider.isEnabled()) {
                return;
            }
            this.currentMouseX = mouseEvent.getX();
            this.currentMouseY = mouseEvent.getY();
            if (!BasicSliderUI.this.isDragging) {
                return;
            }
            BasicSliderUI.this.slider.setValueIsAdjusting(true);
            switch (BasicSliderUI.this.slider.getOrientation()) {
                case 1: {
                    int n = BasicSliderUI.this.thumbRect.height / 2;
                    int a = mouseEvent.getY() - this.offset;
                    int y = BasicSliderUI.this.trackRect.y;
                    int n2 = BasicSliderUI.this.trackRect.y + (BasicSliderUI.this.trackRect.height - 1);
                    int yPositionForValue = BasicSliderUI.this.yPositionForValue(BasicSliderUI.this.slider.getMaximum() - BasicSliderUI.this.slider.getExtent());
                    if (BasicSliderUI.this.drawInverted()) {
                        n2 = yPositionForValue;
                    }
                    else {
                        y = yPositionForValue;
                    }
                    int min = Math.min(Math.max(a, y - n), n2 - n);
                    BasicSliderUI.this.setThumbLocation(BasicSliderUI.this.thumbRect.x, min);
                    BasicSliderUI.this.slider.setValue(BasicSliderUI.this.valueForYPosition(min + n));
                    break;
                }
                case 0: {
                    int n3 = BasicSliderUI.this.thumbRect.width / 2;
                    int a2 = mouseEvent.getX() - this.offset;
                    int x = BasicSliderUI.this.trackRect.x;
                    int n4 = BasicSliderUI.this.trackRect.x + (BasicSliderUI.this.trackRect.width - 1);
                    int xPositionForValue = BasicSliderUI.this.xPositionForValue(BasicSliderUI.this.slider.getMaximum() - BasicSliderUI.this.slider.getExtent());
                    if (BasicSliderUI.this.drawInverted()) {
                        x = xPositionForValue;
                    }
                    else {
                        n4 = xPositionForValue;
                    }
                    int min2 = Math.min(Math.max(a2, x - n3), n4 - n3);
                    BasicSliderUI.this.setThumbLocation(min2, BasicSliderUI.this.thumbRect.y);
                    BasicSliderUI.this.slider.setValue(BasicSliderUI.this.valueForXPosition(min2 + n3));
                    break;
                }
            }
            return;
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
        }
    }
    
    public class ScrollListener implements ActionListener
    {
        int direction;
        boolean useBlockIncrement;
        
        public ScrollListener() {
            this.direction = 1;
            this.direction = 1;
            this.useBlockIncrement = true;
        }
        
        public ScrollListener(int direction, boolean useBlockIncrement) {
            this.direction = 1;
            this.direction = direction;
            this.useBlockIncrement = useBlockIncrement;
        }
        
        public void setDirection(int direction) {
            this.direction = direction;
        }
        
        public void setScrollByBlock(boolean useBlockIncrement) {
            this.useBlockIncrement = useBlockIncrement;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            if (this.useBlockIncrement) {
                BasicSliderUI.this.scrollByBlock(this.direction);
            }
            else {
                BasicSliderUI.this.scrollByUnit(this.direction);
            }
            if (!BasicSliderUI.this.trackListener.shouldScroll(this.direction)) {
                ((Timer)actionEvent.getSource()).stop();
            }
        }
    }
    
    public class ComponentHandler extends ComponentAdapter
    {
        @Override
        public void componentResized(ComponentEvent componentEvent) {
            BasicSliderUI.access$100(BasicSliderUI.this).componentResized(componentEvent);
        }
    }
    
    public class FocusHandler implements FocusListener
    {
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicSliderUI.access$100(BasicSliderUI.this).focusGained(focusEvent);
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            BasicSliderUI.access$100(BasicSliderUI.this).focusLost(focusEvent);
        }
    }
    
    public class ActionScroller extends AbstractAction
    {
        int dir;
        boolean block;
        JSlider slider;
        
        public ActionScroller(JSlider slider, int dir, boolean block) {
            this.dir = dir;
            this.block = block;
            this.slider = slider;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            BasicSliderUI.Actions.access$600(BasicSliderUI.SHARED_ACTION, this.slider, BasicSliderUI.this, this.dir, this.block);
        }
        
        @Override
        public boolean isEnabled() {
            boolean enabled = true;
            if (this.slider != null) {
                enabled = this.slider.isEnabled();
            }
            return enabled;
        }
    }
    
    static class SharedActionScroller extends AbstractAction
    {
        int dir;
        boolean block;
        
        public SharedActionScroller(int dir, boolean block) {
            this.dir = dir;
            this.block = block;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JSlider slider = (JSlider)actionEvent.getSource();
            BasicSliderUI basicSliderUI = (BasicSliderUI)BasicLookAndFeel.getUIOfType(slider.getUI(), BasicSliderUI.class);
            if (basicSliderUI == null) {
                return;
            }
            BasicSliderUI.Actions.access$600(BasicSliderUI.SHARED_ACTION, slider, basicSliderUI, this.dir, this.block);
            return;
        }
    }
    
    private static class Actions extends UIAction
    {
        public static final String POSITIVE_UNIT_INCREMENT = "positiveUnitIncrement";
        public static final String POSITIVE_BLOCK_INCREMENT = "positiveBlockIncrement";
        public static final String NEGATIVE_UNIT_INCREMENT = "negativeUnitIncrement";
        public static final String NEGATIVE_BLOCK_INCREMENT = "negativeBlockIncrement";
        public static final String MIN_SCROLL_INCREMENT = "minScroll";
        public static final String MAX_SCROLL_INCREMENT = "maxScroll";
        
        Actions() {
            super(null);
        }
        
        public Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JSlider slider = (JSlider)actionEvent.getSource();
            BasicSliderUI basicSliderUI = (BasicSliderUI)BasicLookAndFeel.getUIOfType(slider.getUI(), BasicSliderUI.class);
            String name = this.getName();
            if (basicSliderUI == null) {
                return;
            }
            if ("positiveUnitIncrement" == name) {
                this.scroll(slider, basicSliderUI, 1, false);
            }
            else if ("negativeUnitIncrement" == name) {
                this.scroll(slider, basicSliderUI, -1, false);
            }
            else if ("positiveBlockIncrement" == name) {
                this.scroll(slider, basicSliderUI, 1, true);
            }
            else if ("negativeBlockIncrement" == name) {
                this.scroll(slider, basicSliderUI, -1, true);
            }
            else if ("minScroll" == name) {
                this.scroll(slider, basicSliderUI, -2, false);
            }
            else if ("maxScroll" == name) {
                this.scroll(slider, basicSliderUI, 2, false);
            }
            return;
        }
        
        private void scroll(JSlider slider, BasicSliderUI basicSliderUI, int n, boolean b) {
            int inverted = slider.getInverted();
            if (n == -1 || n == 1) {
                if (!(inverted == 0)) {
                    n = ((n == 1) ? -1 : 1);
                }
                if (b) {
                    basicSliderUI.scrollByBlock(n);
                }
                else {
                    basicSliderUI.scrollByUnit(n);
                }
            }
            else {
                if (!(inverted == 0)) {
                    n = ((n == -2) ? 2 : -2);
                }
                slider.setValue((n == -2) ? slider.getMinimum() : slider.getMaximum());
            }
        }
        
        static void access$600(BasicSliderUI.Actions actions, JSlider slider, BasicSliderUI basicSliderUI, int n, boolean b) {
            actions.scroll(slider, basicSliderUI, n, b);
        }
    }
}
